package org.wso2.identity.esb.kerberos;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.neethi.Policy;
import org.apache.neethi.PolicyEngine;
import org.apache.rampart.RampartMessageData;

public class KerberosClient {

	/**
	 * @param args
	 */

	final static String RELYING_PARTY_SERVICE_EPR = "http://192.168.2.13/EchoServices/EchoService.svc";

	/**
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		ConfigurationContext confContext = null;
		Policy servicePolicy = null;
		String trustStore = null;

		// You need to import the Identity Server, public certificate to this key store.
		// By default it's there - if you use wso2carbon.jks from [ESB_HOME]\resources\security
		trustStore = System.getProperty("user.dir") + "\\src\\wso2carbon.jks";
		// We are accessing STS over HTTPS - so need to set trustStore parameters.
		System.setProperty("javax.net.ssl.trustStore", trustStore);
		System.setProperty("javax.net.ssl.trustStorePassword", "wso2carbon");

		// Create configuration context - you will have Rampart module engaged in the
		// client.axis2.xml
		confContext = ConfigurationContextFactory.createConfigurationContextFromFileSystem("repo",
				"repo/conf/client.axis2.xml");

		servicePolicy = loadServicePolicy("policy-1.xml");

		ServiceClient client = new ServiceClient(confContext, null);
		Options options = new Options();
		options.setAction("urn:echoString");
		options.setTo(new EndpointReference(RELYING_PARTY_SERVICE_EPR));
		options.setProperty(RampartMessageData.KEY_RAMPART_POLICY, servicePolicy);
		client.setOptions(options);

		client.engageModule("addressing");
		client.engageModule("rampart");

		OMElement response = client.sendReceive(getPayload("Hello world"));

		System.out.println("Response  : " + response);

	}

	private static Policy loadServicePolicy(String xmlPath) throws Exception {
		StAXOMBuilder builder = null;
		Policy policy = null;

		builder = new StAXOMBuilder(xmlPath);
		policy = PolicyEngine.getPolicy(builder.getDocumentElement());

		return policy;
	}

	private static OMElement getPayload(String value) {
		OMFactory factory = null;
		OMNamespace ns = null;
		OMElement elem = null;
		OMElement childElem = null;

		factory = OMAbstractFactory.getOMFactory();
		ns = factory.createOMNamespace("http://echo.services.core.carbon.wso2.org", "ns1");
		elem = factory.createOMElement("echoString", ns);
		childElem = factory.createOMElement("in", null);
		childElem.setText(value);
		elem.addChild(childElem);
		return elem;
	}
}
